package net.csn;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.websocket.DeploymentException;


/**
 * 
 * Exemple d'inplémentation d'une console pour répondre aux sollicitations du jeu DR2D.
 * 
 * Le projet courrant comporte trois fichiers:
 *  Application.java		=:	le point d'entrée de l'application.
 *  WebSocketClient.java	=:	l'interface d'échange des messages entre le client (la console) et le serveur (le jeu).
 *  WSCnslPlayer.java		=:	l'implémentation de la réponse du joueur au requêtes du serveur de jeu. 
 * 
 * Les prérequis sont :
 *  JDK 1.7 (Java 7) minimum
 *  Maven 3.x.y
 *  Eclipse 4.4 'Luna' avec l'extension "m2eclipse", 
 *  	pour une aide à l'installation, voir: 
 *      http://www.objis.com/formation-java/tutoriel-maven-integration-eclipse-maven-m2eclipse.html#partie2
 *  
 *  Info: si vous avez un problème de dépendance sur wsCnslPlayerLib,
 *        => sur le nom du projet : click contextuel, 'run as'> 'Maven Install'
 *        devrait résoudre le problème.
 * 
 * 
 * @author strat-num.fr/dr2d
 * 
 *
 */
public class Application {

	/**
	 * L'adresse du serveur de jeu. 
	 * ⚠ Dans votre implémentation ce doit être facilement modifiable.
	 */
	static final String _wsUriService = "ws://10.0.1.21:8181/dr2d";
	
    private static final Logger LOGGER = Logger.getLogger(Application.class.getName());
    
    public static void main(String[] args) {
        
        long compteur = 0L;
        try {
            final URI uri = new URI(_wsUriService);
            LOGGER.log(Level.INFO, "Préparation de la connexion au serveur "+uri.toString());
            
            final WebSocketClient wsClientEndPoint = new WebSocketClient( uri );
            if(wsClientEndPoint==null || wsClientEndPoint.session == null) 
            	throw new DeploymentException("Impossible de se connecter à "+uri.toString());

            LOGGER.log(Level.INFO, String.format("Console connectée ... %s", wsClientEndPoint.session.getRequestURI()));
            
            while (true) { // on attend ...
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException ex) {
                	LOGGER.log(Level.WARNING, null, ex);
                }
                compteur++;
                String message = "Yep "+compteur;
                LOGGER.log(Level.INFO, message);
            }
        } catch (DeploymentException | IOException | URISyntaxException ex) {
            LOGGER.log(Level.SEVERE, null, ex);
        }
        
        LOGGER.log(Level.INFO, "Fin de session.");
    }
}
