package net.csn;


import java.awt.Color;
import java.util.ArrayList;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.csn.dr2d.AnRType;
import net.csn.dr2d.Board;
import net.csn.dr2d.Competitor;
import net.csn.dr2d.Position;
import net.csn.dr2d.Movement;
import net.csn.dr2d.Tile;
import net.csn.dr2d.Windrose;
import net.csn.dr2d.wscp.*;

/**
 * <p>
 * Implémentation minimaliste d'une console<br>
 * qui répond à chaque sollicitation du serveur de jeu DR2D.
 * </p>
 * <p>Dernière révision: mars 2016</p>
 * 
 * <p>Consulter <a href="strat-num.fr/dr2d">strat-num.fr/dr2d</a></p>
 * 
 */
//@WebSocketConsolePlayer
public class WSCsnlPlayer 
	extends CnslPlayerAcknowledgement implements CnslPlayerActions {

    private static final Logger LOGGER = Logger.getLogger(WSCsnlPlayer.class.getName());

     String _playerSessionUID;
    /**
     * l'identifiant du joueur [01..08]
     */
    String _shortId;
    
    ArrayList<Board> _Boards = null;
    
    public WSCsnlPlayer(){
    	self(this); // initialisation de l'extention CnslPlayerAcknowledgement
    }
    
 	@Override //@GameInitialisation
	public void onGameInitialisation(final CnslPlayerArgs_GameInitialisation args) {
        LOGGER.log(Level.INFO, String.format("onGameInitialisation ..."));

        LOGGER.log(Level.INFO, String.format("\nIdentifiant unique de session => %s\n",args.playerSessionUID));
        
        // préparation à l'étape suivante ...
		if(_Boards==null) _Boards = new ArrayList<Board>();
		else _Boards.clear();
	}

	@Override //@InitDeclareBoard
	public void onInitDeclareBoard(CnslPlayerArgs_InitDeclareBoard args) {
		LOGGER.log(Level.INFO, String.format("onInitDeclareBoard ..."));
		
		_Boards.add(args.board);				// on ajoute ce plateau à la liste ...
		final int idx = args.board.boardIdx;	// l'index du plateau [0..n]
		final int bW = args.board.boardWidth;	// la largeur du plateau (nord en face)
		final int bH = args.board.boardHeight;	// la profondeur du plateau (nord en face)
		ArrayList<Tile> activeTiles = args.board.boardActiveTiles;
		LOGGER.log(Level.INFO, 
				String.format("\nPlateau n°%s, dimensions: %d x %d\n\t dont %d tuiles complexes.\n"
				, idx, bW, bH, activeTiles.size()));
	}

	@Override //@InitPlayerFeatures
    public void onInitPlayerFeatures(final CnslPlayerArgs_InitPlayerFeatures args) {
        LOGGER.log(Level.INFO, String.format("onInitPlayerFeatures ..."));
        
        args.pseudo = "Mcnd "+_playerSessionUID;	// on choisi un pseudo
		_shortId = args.mcndId;	// on sauvegarde son identifiant de jeu
		// on choisi une couleur de fond (ici sur un décalage de la couleur proposée)
		final int A = (125 + args.argbBackColor.getAlpha())%256;
		final int R = (125 + args.argbBackColor.getRed())%256;
		final int G = (125 + args.argbBackColor.getGreen())%256;
		final int B = (125 + args.argbBackColor.getBlue())%256;
		args.argbBackColor = new Color(R, G, B, A); // juste pour le fun ...
		LOGGER.log(Level.INFO, 
				String.format("\nMécanoïd %s, pseudo: %s, couleur de fond: %sn"
				, args.mcndId, args.pseudo, args.argbBackColor.toString() ));
    }

	@Override //@InitPlayerLocalization
	public void onInitPlayerLocalization(CnslPlayerArgs_InitPlayerLocalization args) {
		LOGGER.log(Level.INFO, String.format("onInitPlayerLocalization ..."));
		
		@SuppressWarnings("unused")
		final ArrayList<Integer> availableTiles = args.availableTiles;	// liste des emplacemets disponibles
		final Position pos						= args.position;		// position proposée
		final Windrose azimuth					= args.azimuth;			// azimut proposé

		// j'accepte la position et l'orientation proposées qui sont:
		LOGGER.log(Level.INFO, String.format("locX: %d, locY: %d, azimuth: %s",
				pos.x, pos.y, azimuth.toString()));
	}

	@Override //@InitStatusOfCompetitors
	public void onInitStatusOfCompetitors(CnslPlayerArgs_InitStatusOfCompetitors args) {
		LOGGER.log(Level.INFO, String.format("onInitStatusOfCompetitors ..."));
		
		final ArrayList<Competitor> competitors = args.competitors; // liste des joueurs
		
		String dump=""; // on affiche la liste des participants ...
		for(Competitor c : competitors){
			dump+=String.format("\t%s", c.toString());
		}
		LOGGER.log(Level.INFO, String.format("Liste des participants:\n%s", dump));
	}

	@Override //@InitLocateArtefacts
	public void onInitLocateArtefacts(CnslPlayerArgs_InitLocateArtefacts args) {
		LOGGER.log(Level.INFO, String.format("onInitLocateArtefacts ..."));
		
		LOGGER.log(Level.INFO, String.format("Mes objectifs sont %s, %s, %s"
				, args.artefacts.get(0).toString()
				, args.artefacts.get(1).toString()
				, args.artefacts.get(2).toString()
				));
	}
	
	@Override //@QuestStart
	public void onQuestStart(CnslPlayerArgs_QuestStart args) {
		LOGGER.log(Level.INFO, String.format("onQuestStart ..."));
	}

	@Override //@RoundRequireCartouche
	public void onRoundRequireCartouche(CnslPlayerArgs_RoundRequireCartouche args) {
		LOGGER.log(Level.INFO, String.format("onRoundRequireCartouche ..."));
		
		// comme exemple, ici on produit 5 mouvements aléatoires ...
		final Random random = new Random();
		for(int m=0; m<5; ++m){
			Movement mov = new Movement();
			// avant ou arrière ?
			int mv = random.nextInt(2); // [0..1]
			AnRType anrMove = mv==0?AnRType.Move_Back:AnRType.Move_Ahead;
			mov.add(anrMove);
			// en face, droite, gauche ou rien ?
			int rot = random.nextInt(4);// [0..3]
            if(rot>0)
            {
                AnRType anrRot = mv == 1 ? AnRType.Move_90Left :
                                 mv == 2 ? AnRType.Move_180Right :
                                       AnRType.Move_90Right;
                mov.add(anrRot);
            }
    		args.cartouche.add(mov); // ajoute les cinq mouvements à la cartouche ...
		}
	}

	@Override //@onRoundRobotMovement
	public void onRoundRobotMovement(CnslPlayerArgs_RoundRobotMovement args) {
		LOGGER.log(Level.INFO, String.format("onRoundRobotMovements ..."));
	}

	@Override //@RoundBoardMovement
	public void onRoundBoardMovement(CnslPlayerArgs_RoundBoardMovement args) {
		LOGGER.log(Level.INFO, String.format("onRoundBoardMovements ..."));
	}

	@Override //@RoundStatusOfCompetitors
	public void onRoundStatusOfCompetitors(CnslPlayerArgs_RoundStatusOfCompetitors args) {
		LOGGER.log(Level.INFO, String.format("onRoundCompetitorsUpdates ..."));	
	}

	@Override //@RoundBoardAction
	public void onRoundBoardAction(CnslPlayerArgs_RoundBoardAction args) {
		LOGGER.log(Level.INFO, String.format("onRoundBoardAction ..."));
		LOGGER.log(Level.INFO, String.format("\t%s(%s)", args.actionName, args.actionArgs));
	}

	@Override //@QuestAchieved
	public void onQuestAchieved(CnslPlayerArgs_QuestAchieved args) {
		LOGGER.log(Level.INFO, String.format("onQuestAchieved ..."));
		
		// exemple de traitement de l'information :
		LOGGER.log(Level.INFO, String.format("\tLes mécanoïds ayant achevés leur quête sont %s", args.questAchieved.toString()));
		if(args.questAchieved.contains(_shortId)){
			if(args.questAchieved.size()>1)
				LOGGER.log(Level.INFO, String.format("%s : Enfin! j'ai fini!", _shortId));
			else
				LOGGER.log(Level.INFO, String.format("%s : Youpi! j'ai gagné!", _shortId));
		}else{
			LOGGER.log(Level.INFO, String.format("%s : Bravo N°%s!", args.questAchieved.get(args.questAchieved.size()-1)));
		}
		//
	}

	/**
	 * Dernier appel du jeu. Permet de libérer toutes les ressources utilisées. 
	 * A partir de là, le serveur rompt la communication. 
	 * 
	 * @param args, les arguments en lecture seule (si present) et ceux qui peuvent être modifiés (si présent)
	 */
	@Override //@GameFinalisation
	public void onGameFinalisation(CnslPlayerArgs_GameFinalisation args) {
		LOGGER.log(Level.INFO, String.format("onGameFinalisation ..."));
		// libère les ressources ...
		_playerSessionUID = null;
		_shortId = null;
		LOGGER.log(Level.INFO, String.format("à bientôt ...."));
	}

    
}



