package net.csn.dr2d;

/**
 * Describe an action or a reaction that can be performed by the mécanoïds
 * 
 * /// 0x10..					=> signalement
 * ///
 * /// 0x21..					=> déplacements
 * /// 0x22.. 0x24.. 0x28..		=> rotations
 * ///
 * /// 0x8...					=> réactions
 * 
 */
public enum AnRType {
	None						( 0x0000 ),		// ni action ni réaction
	
    Light_Off					( 0x1010 ),    // éteint sa DEL de positionement
    Light_On					( 0x1011 ),    // allume sa DEL de positionement
								
    Move_Ahead					( 0x2100 ),		// 00 move 000 north
    Move_Back					( 0x21B4 ),		// B4 move 180 south
							
    Move_180Right				( 0x22B4 ),		// B4+0 move 180 east
    Move_180Left				( 0x22B6 ),		// B4+2
    Move_90Right				( 0x245A ),		// 5A+0 move 90 east
    Move_90Left					( 0x245C ),		// 5A+2
    Move_45Right				( 0x282D ),		// 2D+0 move 45 east
    Move_45Left					( 0x282F ),		// 2D+2
    //							
    Hustle_McndFront			( 0x8110 ),		// heurte un mécanoïd par l'avant
    Hustle_McndRight			( 0x8111 ),
    Hustle_McndRear				( 0x8112 ),
    Hustle_McndLeft				( 0x8114 ),
								
    Hustle_Wall					( 0x8120 ),		// heurte un mur (par l'avant)
							
    Hustle_PusherFront			( 0x8140 ),		// se fait pousser par l'avant
    Hustle_PusherRight			( 0x8141 ),
    Hustle_PusherRear			( 0x8142 ),
    Hustle_PusherLeft			( 0x8144 ),
    //							 		 
    Damage_LaserFront			( 0x8220 ),     // se reçoit le laser par l'avant
    Damage_LaserRight			( 0x8221 ),
    Damage_LaserRear			( 0x8222 ),
    Damage_LaserLeft			( 0x8224 ),
    Damage_Press				( 0x8240 ),
    Damage_Pothole				( 0x8280 ),
    //							 		 
    Conveyor_MakeMoveFront		( 0x8420 ),     // se fait déplacer par l'avant
    Conveyor_MakeMoveRight		( 0x8421 ),
    Conveyor_MakeMoveRear		( 0x8422 ),
    Conveyor_MakeMoveLeft		( 0x8424 ),
    Conveyor_Turn45Right		( 0x8440 ),
    Conveyor_Turn45Left	    	( 0x8440 ),
    Conveyor_Turn90Right		( 0x8480 ),
    Conveyor_Turn90Left 		( 0x8481 ),
    //							 		 
    Enrichment_EnergyX1			( 0x8811 ),
    Enrichment_EnergyX2			( 0x8812 ),
	Enrichment_Frozen			( 0x8820 ),
    Enrichment_Standby			( 0x8822 ),
	Enrichment_WakeUp			( 0x8824 ),
    Enrichment_LawOne			( 0x8841 ),
    Enrichment_LawTwo			( 0x8842 ),
    Enrichment_LawThree			( 0x8843 ),
    Enrichment_QuestAchieved	( 0x8848 ),
    Enrichment_Victory			( 0x8880 );
	
	int _val;
	AnRType(final int val){
		_val = val;
	}
	public int getValue(){return _val;}

	public static AnRType fromDecValue(final int val){
		for(AnRType v : values()){
			if(v.getValue()==val) return v;
		}
		return None;
	}
	public static AnRType fromHexValue(final String hexVal){ // "0x0000"
		try{
			int val = Integer.decode(hexVal);
			for(AnRType v : values()){
				if(v.getValue()==val) return v;
			}
		}catch(NumberFormatException nfe){
			java.lang.System.out.println(nfe);
		}
		return None;
	}
	public String toHexString(){
		final String hex = String.format("0x%04X", getValue());
		return hex;
	}
}
