package net.csn.dr2d.wscp.annotations;

import java.util.LinkedHashSet;
import java.util.Set;

import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.Filer;
import javax.annotation.processing.Messager;
import javax.annotation.processing.ProcessingEnvironment;
import javax.annotation.processing.RoundEnvironment;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.Element;
import javax.lang.model.element.TypeElement;
import javax.lang.model.util.Elements;
import javax.lang.model.util.Types;
import javax.tools.Diagnostic.Kind;


//We need to specify where the annotation processor resides using SPI. 
//We do this by creating a file called javax.annotation.processing.Processor in the META-INF/services directory. 
//The contents of the file should be a list with full qualified class names to the processors with new line as delimiter:
//@AutoService(Processor.class) // This AutoService annotation processor has been developed by Google and generates the META-INF/services/javax.annotation.processing.Processor file.
public class AnnotationProcessor extends AbstractProcessor {

	Types typeUtils;
	Elements elementUtils;
	Filer filer;
	Messager messager;
	  
	@Override
	public synchronized void init(ProcessingEnvironment processingEnv){ 
		super.init(processingEnv);
	    typeUtils = processingEnv.getTypeUtils();
	    elementUtils = processingEnv.getElementUtils();
	    filer = processingEnv.getFiler();
	    messager = processingEnv.getMessager();
	}

	@Override
	public boolean process(Set<? extends TypeElement> annotations,	RoundEnvironment env) {
				
		Messager messager = processingEnv.getMessager();
		for (TypeElement te : annotations) {
		      for (Element e : env.getElementsAnnotatedWith(te)) {
		    	  messager.printMessage(Kind.NOTE, "Printing: " + e.toString());
		      }
	    }
	    return true;
	}

	/**
	 * Here you have to specify for which annotations this annotation processor should be registered for.
	 * Or from JRE7, use 
	 *  @SupportedAnnotationTypes(
	 *    {
	 *      "net.csn.dr2d.wscp.annotations.WebSocketConsolePlayer",
	 *      "net.csn.dr2d.wscp.annotations.InitPlayerFeatures"
	 *    }
	 *  )
	 */
	@Override
	public Set<String> getSupportedAnnotationTypes() { 
		Set<String> annotations = new LinkedHashSet<String>();
	    annotations.add(WebSocketConsolePlayer.class.getCanonicalName());
	    annotations.add(InitPlayerFeatures.class.getCanonicalName());
	    return annotations;
//		return new HashSet<String>(Arrays.asList(
//				"WebSocketConsolePlayer", 
//				"InitPlayerFeatures"
//				));
	}

	/**
	 * To avoid a warning about source version when the annotation processor is run, 
	 * we specify that we handle the latest source version that the current execution environment supports:
	 * Or from JRE7, use 
	 *   @SupportedSourceVersion(SourceVersion.latestSupported())
	 */
	@Override
	public SourceVersion getSupportedSourceVersion() {
		return SourceVersion.latestSupported();
	}
}
